
#ifndef VTKPVSERVERIMPLEMENTATIONRENDERING_EXPORT_H
#define VTKPVSERVERIMPLEMENTATIONRENDERING_EXPORT_H

#ifdef VTKPVSERVERIMPLEMENTATIONRENDERING_STATIC_DEFINE
#  define VTKPVSERVERIMPLEMENTATIONRENDERING_EXPORT
#  define VTKPVSERVERIMPLEMENTATIONRENDERING_NO_EXPORT
#else
#  ifndef VTKPVSERVERIMPLEMENTATIONRENDERING_EXPORT
#    ifdef ServerImplementationRendering_EXPORTS
        /* We are building this library */
#      define VTKPVSERVERIMPLEMENTATIONRENDERING_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVSERVERIMPLEMENTATIONRENDERING_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVSERVERIMPLEMENTATIONRENDERING_NO_EXPORT
#    define VTKPVSERVERIMPLEMENTATIONRENDERING_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVSERVERIMPLEMENTATIONRENDERING_DEPRECATED
#  define VTKPVSERVERIMPLEMENTATIONRENDERING_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVSERVERIMPLEMENTATIONRENDERING_DEPRECATED_EXPORT
#  define VTKPVSERVERIMPLEMENTATIONRENDERING_DEPRECATED_EXPORT VTKPVSERVERIMPLEMENTATIONRENDERING_EXPORT VTKPVSERVERIMPLEMENTATIONRENDERING_DEPRECATED
#endif

#ifndef VTKPVSERVERIMPLEMENTATIONRENDERING_DEPRECATED_NO_EXPORT
#  define VTKPVSERVERIMPLEMENTATIONRENDERING_DEPRECATED_NO_EXPORT VTKPVSERVERIMPLEMENTATIONRENDERING_NO_EXPORT VTKPVSERVERIMPLEMENTATIONRENDERING_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVSERVERIMPLEMENTATIONRENDERING_NO_DEPRECATED
#    define VTKPVSERVERIMPLEMENTATIONRENDERING_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVSERVERIMPLEMENTATIONRENDERING_EXPORT_H */
