
#ifndef VTKPVVTKEXTENSIONSSIL_EXPORT_H
#define VTKPVVTKEXTENSIONSSIL_EXPORT_H

#ifdef VTKPVVTKEXTENSIONSSIL_STATIC_DEFINE
#  define VTKPVVTKEXTENSIONSSIL_EXPORT
#  define VTKPVVTKEXTENSIONSSIL_NO_EXPORT
#else
#  ifndef VTKPVVTKEXTENSIONSSIL_EXPORT
#    ifdef VTKExtensionsSIL_EXPORTS
        /* We are building this library */
#      define VTKPVVTKEXTENSIONSSIL_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVVTKEXTENSIONSSIL_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVVTKEXTENSIONSSIL_NO_EXPORT
#    define VTKPVVTKEXTENSIONSSIL_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVVTKEXTENSIONSSIL_DEPRECATED
#  define VTKPVVTKEXTENSIONSSIL_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVVTKEXTENSIONSSIL_DEPRECATED_EXPORT
#  define VTKPVVTKEXTENSIONSSIL_DEPRECATED_EXPORT VTKPVVTKEXTENSIONSSIL_EXPORT VTKPVVTKEXTENSIONSSIL_DEPRECATED
#endif

#ifndef VTKPVVTKEXTENSIONSSIL_DEPRECATED_NO_EXPORT
#  define VTKPVVTKEXTENSIONSSIL_DEPRECATED_NO_EXPORT VTKPVVTKEXTENSIONSSIL_NO_EXPORT VTKPVVTKEXTENSIONSSIL_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVVTKEXTENSIONSSIL_NO_DEPRECATED
#    define VTKPVVTKEXTENSIONSSIL_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVVTKEXTENSIONSSIL_EXPORT_H */
