
#ifndef VTKCLIENTSERVER_EXPORT_H
#define VTKCLIENTSERVER_EXPORT_H

#ifdef VTKCLIENTSERVER_STATIC_DEFINE
#  define VTKCLIENTSERVER_EXPORT
#  define VTKCLIENTSERVER_NO_EXPORT
#else
#  ifndef VTKCLIENTSERVER_EXPORT
#    ifdef ClientServer_EXPORTS
        /* We are building this library */
#      define VTKCLIENTSERVER_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKCLIENTSERVER_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKCLIENTSERVER_NO_EXPORT
#    define VTKCLIENTSERVER_NO_EXPORT 
#  endif
#endif

#ifndef VTKCLIENTSERVER_DEPRECATED
#  define VTKCLIENTSERVER_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKCLIENTSERVER_DEPRECATED_EXPORT
#  define VTKCLIENTSERVER_DEPRECATED_EXPORT VTKCLIENTSERVER_EXPORT VTKCLIENTSERVER_DEPRECATED
#endif

#ifndef VTKCLIENTSERVER_DEPRECATED_NO_EXPORT
#  define VTKCLIENTSERVER_DEPRECATED_NO_EXPORT VTKCLIENTSERVER_NO_EXPORT VTKCLIENTSERVER_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKCLIENTSERVER_NO_DEPRECATED
#    define VTKCLIENTSERVER_NO_DEPRECATED
#  endif
#endif

#endif /* VTKCLIENTSERVER_EXPORT_H */
