
#ifndef VTKPVSERVERMANAGERAPPLICATION_EXPORT_H
#define VTKPVSERVERMANAGERAPPLICATION_EXPORT_H

#ifdef VTKPVSERVERMANAGERAPPLICATION_STATIC_DEFINE
#  define VTKPVSERVERMANAGERAPPLICATION_EXPORT
#  define VTKPVSERVERMANAGERAPPLICATION_NO_EXPORT
#else
#  ifndef VTKPVSERVERMANAGERAPPLICATION_EXPORT
#    ifdef ServerManagerApplication_EXPORTS
        /* We are building this library */
#      define VTKPVSERVERMANAGERAPPLICATION_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVSERVERMANAGERAPPLICATION_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVSERVERMANAGERAPPLICATION_NO_EXPORT
#    define VTKPVSERVERMANAGERAPPLICATION_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVSERVERMANAGERAPPLICATION_DEPRECATED
#  define VTKPVSERVERMANAGERAPPLICATION_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVSERVERMANAGERAPPLICATION_DEPRECATED_EXPORT
#  define VTKPVSERVERMANAGERAPPLICATION_DEPRECATED_EXPORT VTKPVSERVERMANAGERAPPLICATION_EXPORT VTKPVSERVERMANAGERAPPLICATION_DEPRECATED
#endif

#ifndef VTKPVSERVERMANAGERAPPLICATION_DEPRECATED_NO_EXPORT
#  define VTKPVSERVERMANAGERAPPLICATION_DEPRECATED_NO_EXPORT VTKPVSERVERMANAGERAPPLICATION_NO_EXPORT VTKPVSERVERMANAGERAPPLICATION_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVSERVERMANAGERAPPLICATION_NO_DEPRECATED
#    define VTKPVSERVERMANAGERAPPLICATION_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVSERVERMANAGERAPPLICATION_EXPORT_H */
