
#ifndef VTKRENDERINGVOLUMEAMR_EXPORT_H
#define VTKRENDERINGVOLUMEAMR_EXPORT_H

#ifdef VTKRENDERINGVOLUMEAMR_STATIC_DEFINE
#  define VTKRENDERINGVOLUMEAMR_EXPORT
#  define VTKRENDERINGVOLUMEAMR_NO_EXPORT
#else
#  ifndef VTKRENDERINGVOLUMEAMR_EXPORT
#    ifdef RenderingVolumeAMR_EXPORTS
        /* We are building this library */
#      define VTKRENDERINGVOLUMEAMR_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKRENDERINGVOLUMEAMR_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKRENDERINGVOLUMEAMR_NO_EXPORT
#    define VTKRENDERINGVOLUMEAMR_NO_EXPORT 
#  endif
#endif

#ifndef VTKRENDERINGVOLUMEAMR_DEPRECATED
#  define VTKRENDERINGVOLUMEAMR_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKRENDERINGVOLUMEAMR_DEPRECATED_EXPORT
#  define VTKRENDERINGVOLUMEAMR_DEPRECATED_EXPORT VTKRENDERINGVOLUMEAMR_EXPORT VTKRENDERINGVOLUMEAMR_DEPRECATED
#endif

#ifndef VTKRENDERINGVOLUMEAMR_DEPRECATED_NO_EXPORT
#  define VTKRENDERINGVOLUMEAMR_DEPRECATED_NO_EXPORT VTKRENDERINGVOLUMEAMR_NO_EXPORT VTKRENDERINGVOLUMEAMR_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKRENDERINGVOLUMEAMR_NO_DEPRECATED
#    define VTKRENDERINGVOLUMEAMR_NO_DEPRECATED
#  endif
#endif

#endif /* VTKRENDERINGVOLUMEAMR_EXPORT_H */
