
#ifndef VTKPVCLIENTSERVERCOREPYTHON_EXPORT_H
#define VTKPVCLIENTSERVERCOREPYTHON_EXPORT_H

#ifdef VTKPVCLIENTSERVERCOREPYTHON_STATIC_DEFINE
#  define VTKPVCLIENTSERVERCOREPYTHON_EXPORT
#  define VTKPVCLIENTSERVERCOREPYTHON_NO_EXPORT
#else
#  ifndef VTKPVCLIENTSERVERCOREPYTHON_EXPORT
#    ifdef ClientServerCorePython_EXPORTS
        /* We are building this library */
#      define VTKPVCLIENTSERVERCOREPYTHON_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVCLIENTSERVERCOREPYTHON_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVCLIENTSERVERCOREPYTHON_NO_EXPORT
#    define VTKPVCLIENTSERVERCOREPYTHON_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVCLIENTSERVERCOREPYTHON_DEPRECATED
#  define VTKPVCLIENTSERVERCOREPYTHON_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVCLIENTSERVERCOREPYTHON_DEPRECATED_EXPORT
#  define VTKPVCLIENTSERVERCOREPYTHON_DEPRECATED_EXPORT VTKPVCLIENTSERVERCOREPYTHON_EXPORT VTKPVCLIENTSERVERCOREPYTHON_DEPRECATED
#endif

#ifndef VTKPVCLIENTSERVERCOREPYTHON_DEPRECATED_NO_EXPORT
#  define VTKPVCLIENTSERVERCOREPYTHON_DEPRECATED_NO_EXPORT VTKPVCLIENTSERVERCOREPYTHON_NO_EXPORT VTKPVCLIENTSERVERCOREPYTHON_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVCLIENTSERVERCOREPYTHON_NO_DEPRECATED
#    define VTKPVCLIENTSERVERCOREPYTHON_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVCLIENTSERVERCOREPYTHON_EXPORT_H */
