
#ifndef VTKPVCINEMAREADER_EXPORT_H
#define VTKPVCINEMAREADER_EXPORT_H

#ifdef VTKPVCINEMAREADER_STATIC_DEFINE
#  define VTKPVCINEMAREADER_EXPORT
#  define VTKPVCINEMAREADER_NO_EXPORT
#else
#  ifndef VTKPVCINEMAREADER_EXPORT
#    ifdef CinemaReader_EXPORTS
        /* We are building this library */
#      define VTKPVCINEMAREADER_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define VTKPVCINEMAREADER_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef VTKPVCINEMAREADER_NO_EXPORT
#    define VTKPVCINEMAREADER_NO_EXPORT 
#  endif
#endif

#ifndef VTKPVCINEMAREADER_DEPRECATED
#  define VTKPVCINEMAREADER_DEPRECATED __declspec(deprecated)
#endif

#ifndef VTKPVCINEMAREADER_DEPRECATED_EXPORT
#  define VTKPVCINEMAREADER_DEPRECATED_EXPORT VTKPVCINEMAREADER_EXPORT VTKPVCINEMAREADER_DEPRECATED
#endif

#ifndef VTKPVCINEMAREADER_DEPRECATED_NO_EXPORT
#  define VTKPVCINEMAREADER_DEPRECATED_NO_EXPORT VTKPVCINEMAREADER_NO_EXPORT VTKPVCINEMAREADER_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef VTKPVCINEMAREADER_NO_DEPRECATED
#    define VTKPVCINEMAREADER_NO_DEPRECATED
#  endif
#endif

#endif /* VTKPVCINEMAREADER_EXPORT_H */
