
#ifndef PQAPPLICATIONCOMPONENTS_EXPORT_H
#define PQAPPLICATIONCOMPONENTS_EXPORT_H

#ifdef PQAPPLICATIONCOMPONENTS_STATIC_DEFINE
#  define PQAPPLICATIONCOMPONENTS_EXPORT
#  define PQAPPLICATIONCOMPONENTS_NO_EXPORT
#else
#  ifndef PQAPPLICATIONCOMPONENTS_EXPORT
#    ifdef pqApplicationComponents_EXPORTS
        /* We are building this library */
#      define PQAPPLICATIONCOMPONENTS_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define PQAPPLICATIONCOMPONENTS_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef PQAPPLICATIONCOMPONENTS_NO_EXPORT
#    define PQAPPLICATIONCOMPONENTS_NO_EXPORT 
#  endif
#endif

#ifndef PQAPPLICATIONCOMPONENTS_DEPRECATED
#  define PQAPPLICATIONCOMPONENTS_DEPRECATED __declspec(deprecated)
#endif

#ifndef PQAPPLICATIONCOMPONENTS_DEPRECATED_EXPORT
#  define PQAPPLICATIONCOMPONENTS_DEPRECATED_EXPORT PQAPPLICATIONCOMPONENTS_EXPORT PQAPPLICATIONCOMPONENTS_DEPRECATED
#endif

#ifndef PQAPPLICATIONCOMPONENTS_DEPRECATED_NO_EXPORT
#  define PQAPPLICATIONCOMPONENTS_DEPRECATED_NO_EXPORT PQAPPLICATIONCOMPONENTS_NO_EXPORT PQAPPLICATIONCOMPONENTS_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef PQAPPLICATIONCOMPONENTS_NO_DEPRECATED
#    define PQAPPLICATIONCOMPONENTS_NO_DEPRECATED
#  endif
#endif

#endif /* PQAPPLICATIONCOMPONENTS_EXPORT_H */
